function MV = MinVel(Comp, P, T, wght)
%function MV = MinVel(Comp, Pressure, Temperature, WeightType)
%
%	Written by Oliver Boyd
%	V0.1 Released Nov. 16th 2007
%
%	This program calculates the velocity of the mineral assemblage 
%	given by Comp at a given pressure and temperature (which may be
%	vectors). It returns the Density and P and S wave velocities in the
%	structure MV. The velocities are expressed as Voight, Reuss,
%       and Voight-Reuss-Hill averages.
%
%	The data required for this analysis is taken from Hacker and Abers, 2003.
%	The moduli at pressure and temperature are calculated based on the
%	procedures of Bina and Helffrich, 1992, and Holland and Powell, 1998.
%
%	OUTPUT (SI Units)
%		MV - structure containing
%			Vpv - P-wave velocity, Voight average
%			Vpr - P-wave velocity, Reuss average
%			Vsv - S-wave velocity, Voight average
%			Vsr - S-wave velocity, Reuss average
%			p   - Density
%			a   - Thermal Expansivity
%			Kv  - Bulk modulus, Voight average
%			Gv  - Bulk modulus, Voight average
%			Kr  - Bulk modulus, Reuss average
%			Gr  - Bulk modulus, Reuss average
%			Vp  - P-wave velocity, Voight-Reuss-Hill average
%			Vs  - S-wave velocity, Voight-Reuss-Hill average
%			K   - Bulk modulus, Voight-Reuss-Hill average
%			G   - Bulk modulus, Voight-Reuss-Hill average
%
%	INPUTS
%	Pressure - desired pressure or vector of pressures (Pa)
%	Temperature - desired temperature or vector of temperatures (K)
%	WeightType - type of compositional input. If WeightType is set to 1, the
%		compositions are assumed to be mol% and	the minerals are weighted
%		by their formula unit volume, Unit cell volume	divided by the Z
%		number. If WeightType is zero, compositions are assumed	to be
%		volume. This latter type is currently the preferred method.
%
%	Comp - a composition vector. The index of this vector corresponds to the
%		following minerals. Its' value corresponds to its fraction (0 to 1).
%
%	1. aqz      Alpha quartz
%	2. bqz      Beta quartz
%	3. coe      Coesite
%	4. hAb      High Albite
%	5. lAb      Low Albite
%	6. an       Anorthite
%	7. or       Orthoclase
%	8. san      Sanidine
%	9. alm      Almandine
%	10. gr      Grossular
%	11. py      Pyrope
%	12. fo      Forsterite
%	13. fa      Fayalite
%	14. di      Diopside
%	15. en      Enstatite
%	16. fs      Ferrosilite
%	17. jd      Jadeite
%	18. hed     Hedenbergite
%	19. gl      Glaucophane
%	20. fgl     Ferroglaucophane
%	21. tr      Tremolite
%	22. fact    Ferro-actinolite
%	23. ts      Tschermakite
%	24. parg    Pargasite
%	25. hb      Hornblende
%	26. anth    Anthophyllite
%	27. phl     Phlogopite
%	28. ann     Annite
%	29. mu      Muscovite
%	30. cel
%	31. ta      Talc
%	32. clin    Clinochlore
%	33. daph    Daphnite
%	34. atg     Antigorite
%	35. zo      Zoisite
%	36. cz      Clinozoisite
%	37. ep      Epidote
%	38. law     Lawsonite
%	39. pre     Prehnite 
%	40. pump    Pumpellyite
%	41. lmt     Laumontite
%	42. wrk     Wairakite
%	43. br      Brucite
%	44. chum    Chlinohumite
%	45. phA     Phase A
%	46. sill    Sillimanite
%	47. ky      Kyanite
%	48. sp      Spinel
%	49. herc    Hercynite
%	50. mt      Magnetite
%	51. cc      Calcite
%	52. arag    Aragonite
%	53. cd      Corundum
%

% DA is Duffy and Anderson, 1989
% Wt is a weighting by volume, unit cell volume divided by Z number

Par = dlmread('MinPar.csv',',',0,0);

if abs(sum(Comp) - 1) > 1e-6
	disp('Composition does not sum to one. - Exiting')
	Comp
	sum(Comp)
	pause
	return
end
nPar = length(Par(1,:));
nMin = length(Par(:,1));
nPT = length(P);
p = zeros(nMin,nPT);
Vp = zeros(nMin,nPT);
Vs = zeros(nMin,nPT);
a = zeros(nMin,nPT);

gind = find(Comp > 0);
nMin = length(gind);
for j = 1:nMin
	k = gind(j);
	if Par(k,14) == 0
		Wt(j) = 1;
	else
		Wt(j) = Par(k,13)/Par(k,14);
	end
	cmp(j) = Comp(gind(j));
	[p(j,:), Vp(j,:), Vs(j,:), a(j,:), K(j,:), G(j,:), V(j,:)] = GetV(P,T, ...
		Par(k,11), Par(k,12), Par(k,9), ...
		Par(k,3), Par(k,5), ...
		Par(k,8), Par(k,10), ...
		Par(k,6), Par(k,7), Par(k,2));
end

if wght == 1
	MV = GetMV(p, K, G, Vp, Vs, a, cmp, Wt);
else
	MV = GetMV(p, K, G, Vp, Vs, a, cmp, ones(size(Wt)));
end

return

function [pTP, Vp, Vs, aTP, Ks, GTP, VTP] = GetV(P, T, th, dt, gamma, p, a, G, dGdP, K, dKdP, Vo)

% GetV returns the density, p, P and S velocity, Vp and Vs, and thermal
% expensivity, a, at a given and temperature and pressure, T and P. The postscript
% TP stands for temperature and pressure.
% It requires the following parameters:
% 
%	P - Pressure
%	T - Temperature
%	a - Thermal expansivity
%	p - Density
%	G - Shear Modulus
%	K - Bulk Modulus
%	d.dP - Derivative with respect to pressure
%	dt = isothermal second gruneisen parameter
%	th - first gruneisen parameter
%	gamma - dlnGdp at constant pressure
%	Vo - Volume at STP
%

phi = a*((T-298) - 20*(sqrt(T) - sqrt(298)));
aT = a*(1-1e1./sqrt(T));
pT = p*exp(-phi);
VT = Vo*exp(phi);
KT = K*exp(-dt*phi);
GT = G*exp(-gamma*phi);

f = GetF(P,KT,dKdP);
pP = p*(1 + 2*f).^(3/2);
VTP = VT.*(1 + 2*f).^(-3/2);

%KTP = KT.*((1+2*f).^(5/2)).*(1 - f.*(5 - 3*dKdP));
%GTP = GT.*((1+2*f).^(5/2)).*(1 - f.*(5 - 3*dGdP*KT./GT));

%Assume ddKddP is 1/KT and ddGddP is 1/GT as suggested by Bina and Helffrich, 1992
KTP = KT.*((1+2*f).^(5/2)).*(1 - f.*(5 - 3*dKdP) + (f.^2)*(9 + (3*dKdP - 7)*(3*dKdP - 5))/2);
GTP = GT.*((1+2*f).^(5/2)).*(1 - f.*(5 - 3*dGdP*KT./GT) + (f.^2).*(9*KT.^2./GT.^2 + 9*(dKdP - 4)*dGdP*KT./GT + 35)/2);

aTP = aT.*((pP/p).^-dt);
pTP = pP.*pT/p;
Ks = KTP.*(1 + T*th.*aTP);

Vp = sqrt((Ks + 4*GTP/3)./pTP);
Vs = sqrt(GTP./pTP);

return

function f = GetF(P,KT,dKdP)

% From Bina and Helffrich, 1992. Assume second derivative of K with respect to P,
% ddKddP, is 1/dKdP as suggested by Bina and Helffrich, else
%
% y = (9*dKdP*ddKddP + 4*e*(4 - 3*dKdP) + 5*(3*dKdP - 5))/6;
%
% Took Eq from Bina and Helfrich, Turned into polynomial, and used matlab roots
% command to get the roots of the polynomial. I use solution closest to First
% order solution.
%

e = 0.75*(4-dKdP);
y = (9 + 4*e*(4 - 3*dKdP) + 5*(3*dKdP - 5))/6;

y2 = y^2;
e2 = e^2;
ey = e * y;

C(1) = 32*y2;
C(2) = 80*y2 - 128*ey;
C(3) = 80*y2 - 320*ey + 64*y + 128*e2;
C(4) = 40*y2 - 320*ey + 160*y + 320*e2 - 128*e;
C(5) = 10*y2 - 160*ey + 160*y + 320*e2 - 320*e + 32;
C(6) = y2 - 40*ey + 80*y + 160*e2 - 320*e + 80;
C(7) = -4*ey + 20*y + 40*e2 - 160*e + 80;
C(8) = 2*y + 4*e2 - 40*e + 40;
C(9) = 10 - 4*e;
C(10) = 1;
C(11) = 0;
for j = 1:length(P)
	C(12) = -((P(j)/KT(j))^2)/9;

	F1 = sqrt(-C(12));
	F = roots(C);
	k = find(imag(F) == 0 & abs(F-F1) == min(abs(F-F1)));
	f(j) = F(k(1));
end
return

function MV = GetMV(p, K, G, Vp, Vs, a, cmp, Wt)

% GetMV calculates the Voight-Reuss averages (of velocities but should be moduli
% though VRH will be about the same) of the s and p-wave velocity (Anderson 1997).

MV.Vpv = zeros(1,length(Vp(1,:)));
MV.Vpr = zeros(1,length(Vp(1,:)));
MV.Vsv = zeros(1,length(Vp(1,:)));
MV.Vsr = zeros(1,length(Vp(1,:)));
MV.p = zeros(1,length(Vp(1,:)));
MV.a = zeros(1,length(Vp(1,:)));

WT = sum(Wt.*cmp);

MV.p = (Wt.*cmp)*p(1:length(cmp),:)/WT;
MV.a = (Wt.*cmp)*a(1:length(cmp),:)/WT;

% Voight Average
%MV.Vpv = (Wt.*cmp)*Vp(1:length(cmp),:)/WT;
%MV.Vsv = (Wt.*cmp)*Vs(1:length(cmp),:)/WT;
MV.Kv = (Wt.*cmp)*K(1:length(cmp),:)/WT;
MV.Gv = (Wt.*cmp)*G(1:length(cmp),:)/WT;
MV.Vpv = sqrt((MV.Kv + 4*MV.Gv/3)./MV.p);
MV.Vsv = sqrt(MV.Gv./MV.p);

% Reuss Average
%MV.Vpr = WT./((Wt.*cmp)*(1./Vp(1:length(cmp),:)));
%MV.Vsr = WT./((Wt.*cmp)*(1./Vs(1:length(cmp),:)));
MV.Kr = WT./((Wt.*cmp)*(1./K(1:length(cmp),:)));
MV.Gr = WT./((Wt.*cmp)*(1./G(1:length(cmp),:)));
MV.Vpr = sqrt((MV.Kr + 4*MV.Gr/3)./MV.p);
MV.Vsr = sqrt(MV.Gr./MV.p);

% Voight Reuss Hill Average
MV.Vp = (MV.Vpv + MV.Vpr)/2;
MV.Vs = (MV.Vsv + MV.Vsr)/2;
MV.K = (MV.Kv + MV.Kr)/2;
MV.G = (MV.Gv + MV.Gr)/2;

return
